/* Declaration of header files */

#include <utility.h>
#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "threadpool.h"

#define MAXTHREADS 20

// Define variables

static int panelHandle;
static int poolHandle;
static int numThreads = 0;
static volatile int exiting = 0;

// Define data structure

typedef struct Foo 
{
	double x;
	int ctrlID;
} DataStruct;

// Thread callback function

static int CVICALLBACK MyThreadFunction ();

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	
	if ((panelHandle = LoadPanel (0, "threadpool.uir", PANEL)) < 0)
		return -1;
	
	/* Create thread pool here */

	CmtNewThreadPool (MAXTHREADS, &poolHandle);
	
	DisplayPanel (panelHandle);
	RunUserInterface ();
	DiscardPanel (panelHandle);

	return 0;
}

// Callback for Create Thread button

int CVICALLBACK NewThread (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
    static int top = 24, left = 8;
    static int panelWidth = 0;
    char threadName[50];
    int ctrlID;
    int error = 0;
	DataStruct *a;
	
	a = malloc(sizeof(DataStruct));
	
	// Different data passed to different threads
	
	a->x = numThreads * 2 + 15;

	switch (event)
	{
		case EVENT_COMMIT:
		    
			/* See how many threads we have already created */
			
            if (numThreads == MAXTHREADS) 
			{
                MessagePopup("Warning", "You have created the maximum number of threads for this application");
                return 0;
            }    
            
            /* Create the UIR numeric control */
           
			sprintf(threadName, "Thread %d", numThreads + 1);
            
            if (panelWidth == 0)
                GetPanelAttribute (panel, ATTR_WIDTH, &panelWidth);

            left =  8 + (numThreads % (panelWidth/83)) * (83);
            top  = 30 + (numThreads / (panelWidth/83)) * (48);

            ctrlID = NewCtrl (panel, CTRL_NUMERIC, threadName, top, left);
			a->ctrlID = ctrlID;
            SetCtrlAttribute (panel, ctrlID, ATTR_DATA_TYPE, VAL_UNSIGNED_INTEGER);
            SetCtrlAttribute (panel, ctrlID, ATTR_CTRL_MODE, VAL_INDICATOR);

            /* Schedule new thread function here */
		
			CmtScheduleThreadPoolFunction (poolHandle, MyThreadFunction, a, NULL);
			numThreads++;
			
			break;
		}
	return 0;
}


//-----------------------------------------------------------------------------
// MyThreadFunction - Each started thread will execute this code until request to exit
//-----------------------------------------------------------------------------

static int CVICALLBACK MyThreadFunction (DataStruct *Data)
{
    int threadCount = 0;
	
	SetCtrlVal (panelHandle, PANEL_Parameter, Data->x);
		
    while (!exiting) 
	{
        // Update the thread local value
        threadCount++;
		
		// Show the current thread count
		SetCtrlVal (panelHandle, Data->ctrlID, threadCount);
        
		// Tell system to expire the rest of this thread's time slice
        Delay (0.0001);
    }   
    
    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
        	/* Discard thread pool here and set exiting flag */
			exiting = 1;
			CmtDiscardThreadPool (poolHandle);
			
			QuitUserInterface (0);
			break;
		}
	return 0;
}
